#ifndef HELPERSTUB_GRAPH_H
#define HELPERSTUB_GRAPH_H

/*
Copyright (c) 1997-2025, John M. Boyer
All rights reserved.
See the LICENSE.TXT file for licensing information.
*/

// NOTE: This helper stub has been added for backwards compatibility to
// support downstream consumers who expect graph.h to be at the
// root of the installed planarity headers directory. Future downstream
// consumers are advised to include the graphLib.h helper stub instead
// because it offers access to all features in the graph library of
// the planarity project.

#ifdef __cplusplus
extern "C"
{
#endif

#include "c/graphLib/graph.h"

#ifdef __cplusplus
}
#endif

#endif
